import os

class Configuration:
    def __init__(self, **entries): 
        self.__dict__.update(entries)


def parse_version(args):
    """
    Parse the arguments and extract the version as a dictionary
    """
    versions = {}

    if args.get("--app-version"):
        versions['app'] = args.get("--app-version")
    if args.get("--env-version"):
        versions['env'] = args.get("--env-version")
    if args.get("--service-version"):
        versions['service'] = args.get("--service-version")

    return versions


def parse_project(args):
    """
    Parse the arguments and return a Configuration object for the project.
    """
    dico = {}

    dico['config_file'] = os.path.abspath(os.path.expanduser(args['<config_file>']))
    dico['full_path'] = os.path.dirname(dico['config_file'])

    return Configuration(**dico)


def parse_prepare(args):
    """
    Parse the arguments and return a Configuration object for the preparation (before packaging).
    """
    dico = {}

    # path for building the packages. will be defaulted to temp folder if none.
    dico['build_path'] = args.get("--build")
    dico['architecture'] = args.get("--arch")
    dico['pip_options'] = args.get("--pip-options")
    dico['extra_description'] = args.get("--extra-description")

    return Configuration(**dico)

def parse_package(args):
    """
    """
    dico = {}

    dico['web_callback_url'] = args.get("--web-callback")
    dico['debs_path'] = args.get("--output") if args.get("--output") else os.getcwd()

    return Configuration(**dico)


import json
import requests

def jsonfy_pkg(pkg):
	"""
	Transform the package in a correct json string for callback.
	"""
	return {"name": pkg.full_package_name, "version": pkg.version, "file_name": pkg.final_deb_name}

def do_web_callback(web_callback_url, pkg):
    """
    Make a POST HTTP request to callback url for each package.
    Body of the request should be json data:
    {"name": "foo", "version": "0.0.0", "file_name": "foo-0.0.0-all.deb"}
    """
    payload = jsonfy_pkg(pkg)
    r = requests.post(web_callback_url, data=json.dumps(payload), headers={'content-type': 'application/json'})
    return r.status_code==200



import yaml


def get_yaml_config(config_file, env="default"):
    # open config file and load yaml.
    stream = file(config_file, 'r')
    data = yaml.load(stream)

    # if no default, then move on
    if 'default' not in data:
        yield data
    else:
        yield data['default']

        for key, value in data.iteritems():
            if key != 'default':
                yield _merge(data[key], data['default'])


def load_yaml_config(config_file, env="default"):
    """ Read a yaml file and extract the configuration. """

    # open config file and load yaml.
    stream = file(config_file, 'r')
    data = yaml.load(stream)

    # if no default, then move on
    if 'default' not in data:
        return data

    # return the value for the right environment.
    if env == "default":
        final_data = data["default"]
    else:
        final_data = _merge(data[env], data['default'])

    return final_data


def _merge(params, default):
    """ Deep merge of dictionary. """
    if isinstance(params,dict) and isinstance(default,dict):
        for k,v in default.iteritems():
            if k not in params:
                params[k] = v
            else:
                params[k] = _merge(params[k],v)
    return params


def print_pretty(d, indent=0):
    """ Pretty print of nested dictionary """
    for key, value in d.iteritems():
        print '\t' * indent + str(key)
        if isinstance(value, dict):
            print_pretty(value, indent+1)
        else:
            print '\t' * (indent+1) + str(value)


def flatten_dict(init, lkey=''):
    ret = {}
    for rkey,val in init.items():
        key = lkey+rkey
        if type(val) is dict:
            ret.update(flatten_dict(val, key+'_'))
        else:
            ret[key] = val
    return ret



import os
from fabric.api import local

from .utils import local_template, prepare_folder

def get_custom_file(type_, name, template, target_path=None, target_extension=None,
                target_is_executable=None, options=None):
    """ factory for CustomFiles """
    if type_ == 'nginx':
        return NGINXCustomFile(name, template, options=options)
    elif type_ == "binary":
        return BinaryCustomFile(name, template, options=options)
    elif type_ == "debian":
        return DEBIANCustomFile(name, template, options=options)
    elif type_ == "product":
        return ProductCustomFile(name, template, options=options)
    elif type_ == "upstart":
        return UpstartCustomFile(name, template, options=options)
    elif type_ == "newrelic_config":
        return NewrelicConfigCustomFile(name, template, options=options)
    elif type_ == "custom":
        return CustomFile(name, template, target_path, target_extension, target_is_executable, options)
    else:
        raise NotImplementedError


class CustomFile(object):
    is_executable = False
    relative_path = ""
    extension = ""

    def __init__(self, name, template=None, target_path=None, target_extension=None,
                target_is_executable=None, options=None):
        self.name = name
        self.template = template

        if target_is_executable:
            self.is_executable = target_is_executable

        if target_path:
            self.relative_path = target_path

        if target_extension:
            self.extension = target_extension

        self.options = options if options else {}

    def build(self, base_path, options=None, extra_template_dir=None):
        print "{%s}: building file %s" % (self.__class__.__name__, self.name)

        # fuse the path, even if relative_filepath starts with "/"
        full_path = os.path.join(base_path, *self.relative_filepath.split("/"))
        print full_path
        prepare_folder(os.path.dirname(full_path))

        params = {'full_name': self.name}
        if self.options:
            params.update(self.options)
        if options:
            params.update(options)

        my_cwd = os.path.abspath(os.path.dirname(__file__))
        template_dir = [os.path.join(my_cwd, "..", "templates")]
        if extra_template_dir:
            template_dir.append(extra_template_dir)

        local_template(self.template,
            full_path,
            context=params, 
            use_jinja=True, 
            backup=False, 
            template_dir=template_dir)

        if self.is_executable:
            local("chmod +x %s" % full_path)

        self.post_build(full_path)

    def post_build(self, full_path):
        pass

    @property
    def relative_filepath(self):
        return os.path.join(self.relative_path, 
                            "%s%s" % (self.name, self.extension))


class NGINXCustomFile(CustomFile):
    relative_path = os.path.join("etc", "nginx", "sites-available")
    extension = ".conf"


class BinaryCustomFile(CustomFile):
    relative_path = os.path.join("opt", "trebuchet", "bin")
    is_executable = True


class DEBIANCustomFile(CustomFile):
    relative_path = os.path.join("DEBIAN")

    def post_build(self, full_path):
        local('echo "" >> %s' % full_path)
        local('chmod 0775 %s' % full_path)


class ProductCustomFile(CustomFile):
    relative_path = os.path.join("etc", "trebuchet")
    extension = ".conf"

class UpstartCustomFile(CustomFile):
    relative_path = os.path.join("etc", "init")
    extension = ".conf"

# class HelicomServerConfigCustomFile(CustomFile):
#     relative_path = os.path.join("etc", "trebuchet", "helicom_server")
#     extension = ".cfg"

# class HelicomBrokerConfigCustomFile(CustomFile):
#     relative_path = os.path.join("etc", "trebuchet", "helicom_broker")
#     extension = ".cfg"

class NewrelicConfigCustomFile(CustomFile):
    relative_path = os.path.join("etc", "trebuchet", "newrelic")
    extension = ".ini"
